// VL_anglestand-v1.5.lua 
//
// This script prevents player characters from autoturning to face away from the camera when idle.
//
// Modified from Krabs' Anglestand v1
// Modified by Stonecutter

freeslot("S_PLAY_ANGLEIDLE")
states[S_PLAY_ANGLEIDLE] = {SPR_PLAY, SPR2_WAIT, 16, nil, 0, 0, S_PLAY_ANGLEIDLE}


local autoload = true -- Set this to false if you do not want Anglestand to load your settings automatically.
local idleDelay = 3 * TICRATE

-- multiplayer variables
local strafeAnglestand = true
local angleFallAnim = false
local angleAutosave = true

local initialized = false

-- settings saved in AnglestandData.txt
-- line 1 represents wether or not strafe players are prevented from autoturning when standing still.
-- line 2 represents whether Anglestand should automatically save changes to your settings.
-- line 3 represents wether or not the ground falling out from under you or flinging you into the air cancels idle animations.

-- Thanks to birbhorse (EeveeEuporia) for marking Many Minor Tweaks re-useable. I looked its v5.1 i/o code to get started on Anglestand's i/o.
local function SaveData(player)
	if(multiplayer)
		CONS_Printf(player, "Anglestand can only save settings in singleplayer.")
	else
		local anglestandPersistantData = io.openlocal("AnglestandData/booleans.txt", "w")
		
		if not(strafeAnglestand)
			anglestandPersistantData:write("false")
		else
			anglestandPersistantData:write("true")
		end
		
		anglestandPersistantData:write("\n")
		
		if not(angleAutosave)
			anglestandPersistantData:write("false")
		else
			anglestandPersistantData:write("true")
		end
		
		anglestandPersistantData:write("\n")
		
		if not(angleAutosave)
			anglestandPersistantData:write("false")
		else
			anglestandPersistantData:write("true")
		end
	
		anglestandPersistantData:close()
		CONS_Printf(player, "Anglestand settings saved.")
	end
end

local function LoadData(player)
	if(multiplayer)
		CONS_Printf(player, "Anglestand can only load saved settings in singleplayer.")
	else
		local anglestandPersistantData = io.openlocal("AnglestandData/booleans.txt", "r")
		if(anglestandPersistantData == nil)
			CONS_Printf(player, "No Anglestand settings file found. Creating Anglestand settings file.")
			SaveData(player)
			return
		end
	
		local inData = anglestandPersistantData:read("*l")
		
		if(inData == "true")
			strafeAnglestand = true
		else
			strafeAnglestand = false
		end
		
		inData = anglestandPersistantData:read("*l")
		if(inData == "true")
			angleAutosave = true
		else
			angleAutosave = false
		end
		
		inData = anglestandPersistantData:read("*l")
		if(inData == "true")
			angleFallAnim = true
		else
			angleFallAnim = false
		end
	
		anglestandPersistantData:close()
		CONS_Printf(player, "Anglestand settings loaded.")
		if(strafeAnglestand == true)
			CONS_Printf(player, "Anglestand will affect you when in strafe mode.")
		else
			CONS_Printf(player, "Anglestand will not affect you when in strafe mode.")
		end
		
		if(angleFallAnim == true)
			CONS_Printf(player, "Leaving the ground cancels your idle animation.")
		else
			CONS_Printf(player, "Leaving the ground does not cancel your idle animation.")
		end
		
		if(angleAutosave == true)
			CONS_Printf(player, "Anglestand will save whenever you change the Anglestand settings.")
		else
			CONS_Printf(player, "Anglestand will only save when you use the command 'angle_save'")
		end
	end
end

local function ToggleStrafeAngle(player)
	if(multiplayer and not (IsPlayerAdmin(player)))
		CONS_Printf(player, "You must be an admin to use this in multiplayer.")
		return
	end
	if(strafeAnglestand)
		strafeAnglestand = false
		CONS_Printf(player, "Anglestand will not affect you when in strafe mode.")
	else
		strafeAnglestand = true
		CONS_Printf(player, "Anglestand will affect you when in strafe mode.")
	end
	if(angleAutosave)
		SaveData(player)
	end
end

local function ToggleFallAnim(player)
	if(multiplayer and not (IsPlayerAdmin(player)))
			CONS_Printf(player, "You must be an admin to use this in multiplayer.")
			return
	end
	if(angleFallAnim)
		angleFallAnim = false
		CONS_Printf(player, "Leaving the ground does not cancel your idle animation.")
	else
		angleFallAnim = true
		CONS_Printf(player, "Leaving the ground cancels your idle animation.")
	end
	if(angleAutosave)
		SaveData(player)
	end
end

local function ToggleAutosave(player)
	if(multiplayer)
		CONS_Printf(player, "This only works in singleplayer.")
	elseif(angleAutosave)
		angleAutosave = false
		CONS_Printf(player, "Anglestand will only save when you use the command 'angle_save'")
	else
		CONS_Printf(player, "Anglestand will save whenever you change the Anglestand settings.")
		angleAutosave = true
		SaveData(player)
	end
end

local function SettingsDisplay(player)
	CONS_Printf(player, "Anglestand will affect you while in Strafe mode: " + strafeAnglestand)
	CONS_Printf(player, "Anglestand autosaves: " + player.angleAutosave)
	CONS_Printf(player, "Your idle animation only plays on the ground: " + angleFallAnim)
end

COM_AddCommand("angle_strafe", ToggleStrafeAngle, 0)
COM_AddCommand("angle_fall", ToggleFallAnim, 0) -- toggles whether or not detatching from the ground will cancel idle animations in the absence of player input, damage taken, etc.
COM_AddCommand("angle_load", LoadData, 0)
COM_AddCommand("angle_save", SaveData, 0)
COM_AddCommand("angle_autosave", ToggleAutosave, 0)
COM_AddCommand("angle_settings", SettingsDisplay, 0) -- dislpays current Anglestand settings


addHook("PlayerSpawn", function(player)
	if(multiplayer)
		return
	end
	if(initialized == false)
		if(multiplayer or autoload == true)
			LoadData(player)
			initialized = true
		else
			initialized = true
		end
	end
end)


-- fixes problems with Tails' tails disappearing
addHook("FollowMobj", function(player, followMob)
	local mo = player.mo
	if(mo.state == S_PLAY_ANGLEIDLE)
		local newAngle = player.drawangle
		if(newAngle != nil)
			P_TeleportMove(followMob, (mo.x - cos(newAngle)), (mo.y - sin(newAngle)), mo.z)
			followMob.angle = newAngle
			return true
		end
	end
end, MT_TAILSOVERLAY)

addHook("MobjThinker", function(mo)
	local player = mo.player
		if player.playerstate == PST_LIVE
		
			-- prevents script from affecting strafe mode players
			-- only works in singleplayer
			if(strafeAnglestand == false and (not (player.pflags & !PF_FORCESTRAFE)))
				if(mo.state == S_PLAY_ANGLEIDLE)
					mo.state = S_PLAY_WAIT
					player.idlewait = 0
				end
				return
			end
			
			-- If nonIdle is true, later code will know not to trigger an idle animation.
			local nonIdle = 0
			if(player.powers[pw_super] or player.hypermysticsonic)
				nonIdle = 1
			end
			local initialState = mo.state
			local followMob = mo.followmobj
			if player.prevdrawangle
			
				-- special code for telekinetic hedgehogs
				if(mo.skin == "silver")
					if(mo.silv_telekinesis and #mo.silv_telekinesis)	-- Silver was, and might still be, holding something.
						mo.wasTelekinesis = true
						if(mo.silv_thrown and #mo.silv_thrown and not (#mo.silv_telekinesis > #mo.silv_thrown)) -- Silver isn't holding anything anymore.
							player.idlewait = 0
							mo.state = S_PLAY_STND
							mo.wasTelekinesis = false
						end
					elseif(mo.wasTelekinesis == true)	-- Silver was holding something, but isn't anymore.
						player.idlewait = 0
						mo.state = S_PLAY_STND
						mo.wasTelekinesis = false
					elseif(initialState == S_PLAY_WAIT)	-- Silver is in his idle animation. Because he only uses 1 frame for himself, he doesn't need to be using the custom idle state.
						player.drawangle = player.prevdrawangle
						mo.state = S_PLAY_WAIT			-- As a matter of fact, the custom idle state breaks his idle animation.
						player.idlewait = idleDelay
					elseif(initialState == S_PLAY_STND and (not (player.powers[pw_carry] & CR_MINECART)))
			
						player.drawangle = player.prevdrawangle
					
						-- initiates idle animation when the idlewait timer reaches its limit
							if(player.idlewait != nil)
								local idlewait = player.idlewait
								if(idlewait < idleDelay)
									player.idlewait = $ + 1
								else
									if(idlewait < idleDelay + 1)
										mo.state = S_PLAY_WAIT
									end
								end
						
							else
								player.idlewait = 1
							end
					elseif
					(
						mo.momx == 0
						and mo.momy == 0
						and
						(
							initialState == S_PLAY_JUMP
							or initialState == S_PLAY_EDGE
							or initialState == S_PLAY_FALL
							or initialState == S_PLAY_SPRING
							or initialState == S_PLAY_GASP
							or initialState == S_PLAY_FLOAT
						)
					)
						player.drawangle = player.prevdrawangle
						player.idlewait = 0
						
					elseif(initialState == S_PLAY_ANGLEIDLE) -- Fixes bug arising from swapping skins while in S_PLAY_ANGLEIDLE
						mo.state = S_PLAY_STND
						player.idlewait = idleDelay
						
					-- resets idle timer
					else
						player.idlewait = 0
					end
					-- end of Silver's code
					
				-- special code for Dirk
				elseif(mo.skin == "dirk")
					if(mo.dirk_idle >= 0)	-- Dirk is idling
						player.drawangle = player.prevdrawangle
						player.idlewait = idleDelay
					elseif(initialState == S_PLAY_STND and (not (player.powers[pw_carry] & CR_MINECART)))
			
						player.drawangle = player.prevdrawangle
					
						-- initiates idle animation when the idlewait timer reaches its limit
							if(player.idlewait != nil)
								local idlewait = player.idlewait
								if(idlewait < idleDelay)
									player.idlewait = $ + 1
								else
									if(idlewait < idleDelay + 1)
										mo.state = S_PLAY_WAIT
									end
								end
						
							else
								player.idlewait = 1
							end
					elseif
					(
						mo.momx == 0
						and mo.momy == 0
						and
						(
							initialState == S_PLAY_JUMP
							or initialState == S_PLAY_EDGE
							or initialState == S_PLAY_FALL
							or initialState == S_PLAY_SPRING
							or initialState == S_PLAY_GASP
							or initialState == S_PLAY_FLOAT
						)
					)
						player.drawangle = player.prevdrawangle
						player.idlewait = 0
					
					elseif(initialState == S_PLAY_ANGLEIDLE) -- Fixes bug arising from swapping skins while in S_PLAY_ANGLEIDLE
						mo.state = S_PLAY_WAIT
						player.idlewait = idleDelay
					
					-- resets idle timer
					else
						player.idlewait = 0
					end
				-- end of Dirk's code
				
				-- For characters who shouldn't be going into an idle animation.
				elseif(nonIdle == 1)
					if
					(
						initialState == S_PLAY_STND and (not (player.powers[pw_carry] & CR_MINECART))
						or
						(
							mo.momx == 0
							and mo.momy == 0
							and
							(
								initialState == S_PLAY_JUMP
								or initialState == S_PLAY_EDGE
								or initialState == S_PLAY_FALL
								or initialState == S_PLAY_SPRING
								or initialState == S_PLAY_GASP
								or initialState == S_PLAY_FLY
								or initialState == S_PLAY_FLY_TIRED
								or initialState == S_PLAY_GLIDE_LANDING
								or initialState == S_PLAY_SWIM
								or initialState == S_PLAY_TWINSPIN
								or initialState == S_PLAY_BOUNCE
								or initialState == S_PLAY_BOUNCE_LANDING
								or initialState == S_PLAY_FLOAT
							)
						)
					)
						player.drawangle = player.prevdrawangle
					end
				
				-- for everyone else
				-- swaps the default idle animation for the custom one, unless the skin is silver
				elseif (initialState == S_PLAY_WAIT)
					mo.state = S_PLAY_ANGLEIDLE
					player.idlewait = idleDelay + 1
					
				elseif ((initialState == S_PLAY_STND and (not (player.powers[pw_carry] & CR_MINECART))) or initialState == S_PLAY_ANGLEIDLE)
				
					player.drawangle = player.prevdrawangle
					
					-- initiates idle animation when the idlewait timer reaches its limit
					if(player.idlewait != nil and (nonIdle == 0) )
						local idlewait = player.idlewait
						if(idlewait < idleDelay)
							player.idlewait = $ + 1
						else
							if(idlewait < idleDelay + 1)
								mo.state = S_PLAY_ANGLEIDLE
								player.idlewait = $ + 1
							end
						end
						
					else
						player.idlewait = 1
					end
				
				-- prevents autoturning when jumping, teetering, springing, or using a vanilla charcter air ability in place, then resets idle timer
				elseif
				(
					mo.momx == 0
					and mo.momy == 0
					and
					( --#### Note to self: look into alternate ways of doing this for the possibility of supporting custom abilities.
						initialState == S_PLAY_JUMP
						or initialState == S_PLAY_EDGE
						or initialState == S_PLAY_FALL
						or initialState == S_PLAY_SPRING
						or initialState == S_PLAY_GASP
						or initialState == S_PLAY_FLY
						or initialState == S_PLAY_FLY_TIRED
						or initialState == S_PLAY_GLIDE_LANDING
						or initialState == S_PLAY_SWIM
						or initialState == S_PLAY_TWINSPIN
						or initialState == S_PLAY_BOUNCE
						or initialState == S_PLAY_BOUNCE_LANDING
						or initialState == S_PLAY_FLOAT
						
						or (player.bunsFlightTime != nil and player.bunsFlightBegun != nil and player.bunsFlightTime > 0 and player.bunsFlightBegun == true)	-- Bunnie compatability
					)
				)
					player.drawangle = player.prevdrawangle
					player.idlewait = 0
					
				-- resets idle timer
				else
					player.idlewait = 0
				end
			end
			
		player.prevdrawangle = player.drawangle -- sets the angle for the player to maintain next frame
		
		if (initialState == S_PLAY_ANGLEIDLE and (player.rmomx != 0 or player.rmomy != 0))
			mo.state = S_PLAY_WALK		-- This line cancels idle animation when the player moves.
		elseif(initialState == S_PLAY_ANGLEIDLE and angleFallAnim == true and (not P_IsObjectOnGround(mo)) and (not (player.powers[pw_carry] == CR_ROLLOUT)))	--Falling Animation compatability
			if(mo.momz * P_MobjFlip(mo) < 0)
				mo.state = S_PLAY_FALL
			else
				mo.state = S_PLAY_SPRING
			end
		end
	end
		
end, MT_PLAYER)

-- resets everything when the map changes
addHook("MapChange", function()
	for player in players.iterate()
		if((not multiplayer) and not initialized)
			LoadData(player)
			initialized = true
		end
		player.prevdrawangle = nil
		player.idlewait = 0
	end
end)